@@ -47,6 +47,9 @@ predict.egame122 <- function(object, newdata, probs = c("outcome", "action"), ..
 
 sbi122 <- function(y, regr, link)
 {
+    names(regr) <- character(length(regr))
+    names(regr)[1:6] <- c("X1", "X2", "X3", "X4", "Z2", "Z4")
+
     if (link == "probit") {
         fam <- binomial(link = "probit")
     } else {
@@ -145,6 +148,9 @@ actionsToOutcomes122 <- function(probs, log.p = TRUE)
 
 logLik122 <- function(b, y, regr, link, type, ...)
 {
+    names(regr) <- character(length(regr))
+    names(regr)[1:6] <- c("X1", "X2", "X3", "X4", "Z2", "Z4")
+
     probs <- makeProbs122(b, regr, link, type)
     logProbs <- actionsToOutcomes122(probs, log.p = TRUE)
     ans <- logProbs[cbind(1:nrow(logProbs), y)]
@@ -153,6 +159,9 @@ logLik122 <- function(b, y, regr, link, type, ...)
 
 logLikGrad122 <- function(b, y, regr, link, type, ...)
 {
+    names(regr) <- character(length(regr))
+    names(regr)[1:6] <- c("X1", "X2", "X3", "X4", "Z2", "Z4")
+
     u <- makeUtils(b, regr, nutils = 6,
                    unames = c("u11", "u12", "u13", "u14", "u22", "u24"))
     p <- makeProbs122(b, regr, link, type)
@@ -316,6 +325,9 @@ makeResponse122 <- function(yf)
 ##' one be estimated for each player?
 ##' @param boot integer: number of bootstrap iterations to perform (if any).
 ##' @param bootreport logical: whether to print status bar during bootstrapping.
+##' @param profile output from running \code{\link{profile.game}} on a previous
+##' fit of the model, used to generate starting values for refitting when an
+##' earlier fit converged to a non-global maximum.
 ##' @param ... other arguments to pass to the fitting function (see
 ##' \code{\link{maxBFGS}})
 ##' @return An object of class \code{c("game", "egame122")}.  See
@@ -363,6 +375,7 @@ egame122 <- function(formulas, data, subset, na.action,
                      sdByPlayer = FALSE,
                      boot = 0,
                      bootreport = TRUE,
+                     profile,
                      ...)
 {
     cl <- match.call()
@@ -424,20 +437,22 @@ egame122 <- function(formulas, data, subset, na.action,
     regr <- list()
     for (i in seq_len(length(formulas)[2]))
         regr[[i]] <- model.matrix(formulas, data = mf, rhs = i)
-    names(regr) <- character(length(regr))
-    names(regr)[1:6] <- c("X1", "X2", "X3", "X4", "Z2", "Z4")
     rcols <- sapply(regr, ncol)
 
     ## starting values
-    if (startvals == "zero") {
-        sval <- rep(0, sum(rcols))
-    } else if (startvals == "unif") {
-        if (!hasArg(unif))
-            unif <- c(-1, 1)
-        sval <- runif(sum(rcols), unif[1], unif[2])
+    if (missing(profile) || is.null(profile)) {
+        if (startvals == "zero") {
+            sval <- rep(0, sum(rcols))
+        } else if (startvals == "unif") {
+            if (!hasArg(unif))
+                unif <- c(-1, 1)
+            sval <- runif(sum(rcols), unif[1], unif[2])
+        } else {
+            sval <- sbi122(y, regr, link)
+            sval <- c(sval, rep(0, sum(rcols) - length(sval)))
+        }
     } else {
-        sval <- sbi122(y, regr, link)
-        sval <- c(sval, rep(0, sum(rcols) - length(sval)))
+        sval <- svalsFromProfile(profile)
     }
 
     ## identification check
@@ -485,7 +500,8 @@ egame122 <- function(formulas, data, subset, na.action,
     ans$log.likelihood <-
         logLik122(results$estimate, y = y, regr = regr, link = link, type = type)
     ans$call <- cl
-    ans$convergence <- list(code = results$code, message = results$message)
+    ans$convergence <- list(code = results$code, message = results$message,
+                            gradient = !is.null(gr))
     ans$formulas <- formulas
     ans$link <- link
     ans$type <- type
